package javax.swing.text;

import java.beans.PropertyChangeEvent;
import java.awt.KeyboardFocusManager;
import java.awt.AWTKeyStroke;
import java.util.Set;
import java.beans.PropertyChangeListener;
import java.awt.Container;
import javax.swing.SwingUtilities;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Graphics;
import java.awt.Component;

public class ComponentView extends View
{
    private Component createdC;
    private ComponentView.Invalidator c;
    
    public ComponentView(Element element) {
        super(element);
    }
    
    protected Component createComponent() {
        return StyleConstants.getComponent(this.getElement().getAttributes());
    }
    
    public final Component getComponent() {
        return this.createdC;
    }
    
    @Override
    public void paint(Graphics graphics, Shape shape) {
        if (this.c != null) {
            Rectangle rectangle = (shape instanceof Rectangle) ? ((Rectangle)shape) : shape.getBounds();
            this.c.setBounds(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
        }
    }
    
    @Override
    public float getPreferredSpan(int i) {
        if (i != 0 && i != 1) {
            throw new IllegalArgumentException("Invalid axis: " + i);
        }
        if (this.c != null) {
            Dimension preferredSize = this.c.getPreferredSize();
            if (i == 0) {
                return (float)preferredSize.width;
            }
            return (float)preferredSize.height;
        }
        else {
            return 0.0f;
        }
    }
    
    @Override
    public float getMinimumSpan(int i) {
        if (i != 0 && i != 1) {
            throw new IllegalArgumentException("Invalid axis: " + i);
        }
        if (this.c != null) {
            Dimension minimumSize = this.c.getMinimumSize();
            if (i == 0) {
                return (float)minimumSize.width;
            }
            return (float)minimumSize.height;
        }
        else {
            return 0.0f;
        }
    }
    
    @Override
    public float getMaximumSpan(int i) {
        if (i != 0 && i != 1) {
            throw new IllegalArgumentException("Invalid axis: " + i);
        }
        if (this.c != null) {
            Dimension maximumSize = this.c.getMaximumSize();
            if (i == 0) {
                return (float)maximumSize.width;
            }
            return (float)maximumSize.height;
        }
        else {
            return 0.0f;
        }
    }
    
    @Override
    public float getAlignment(int n) {
        if (this.c != null) {
            switch (n) {
                case 0: {
                    return this.c.getAlignmentX();
                }
                case 1: {
                    return this.c.getAlignmentY();
                }
                default: {
                    break;
                }
            }
        }
        return super.getAlignment(n);
    }
    
    @Override
    public void setParent(View parent) {
        super.setParent(parent);
        if (SwingUtilities.isEventDispatchThread()) {
            this.setComponentParent();
        }
        else {
            SwingUtilities.invokeLater(new Runnable() {
                @Override
                public void run() {
                    Document document = ComponentView.this.getDocument();
                    try {
                        if ((document instanceof AbstractDocument)) {
                            ((AbstractDocument)document).readLock();
                        }
                        ComponentView.this.setComponentParent();
                        Container container = ComponentView.this.getContainer();
                        if (container != null) {
                            ComponentView.this.preferenceChanged(null, true, true);
                            container.repaint();
                        }
                    }
                    finally {
                        if ((document instanceof AbstractDocument)) {
                            ((AbstractDocument)document).readUnlock();
                        }
                    }
                }
            });
        }
    }
    
    void setComponentParent() {
        if (this.getParent() != null) {
            Container container = this.getContainer();
            if (container != null) {
                if (this.c == null) {
                    Component component = this.createComponent();
                    if (component != null) {
                        this.createdC = component;
                        this.c = new ComponentView.Invalidator(component);
                    }
                }
                if (this.c != null && this.c.getParent() == null) {
                    container.add(this.c, this);
                    container.addPropertyChangeListener("enabled", this.c);
                }
            }
        }
        else if (this.c != null) {
            Container parent = this.c.getParent();
            if (parent != null) {
                parent.remove(this.c);
                parent.removePropertyChangeListener("enabled", this.c);
            }
        }
    }
    
    @Override
    public Shape modelToView(int i, Shape shape, Position.Bias bias) throws BadLocationException {
        int startOffset = this.getStartOffset();
        int endOffset = this.getEndOffset();
        if (i >= startOffset && i <= endOffset) {
            Rectangle bounds = shape.getBounds();
            if (i == endOffset) {
                bounds.x += bounds.width;
            }
            bounds.width = 0;
            return bounds;
        }
        else {
            throw new BadLocationException(i + " not in range " + startOffset + "," + endOffset, i);
        }
    }
    
    @Override
    public int viewToModel(float n, float n2, Shape shape, Position.Bias[] array) {
        Rectangle rectangle = (Rectangle)shape;
        if (n < (float)(rectangle.x + rectangle.width / 2)) {
            array[0] = Position.Bias.Forward;
            return this.getStartOffset();
        }
        array[0] = Position.Bias.Backward;
        return this.getEndOffset();
    }
    
    class Invalidator extends Container implements PropertyChangeListener
    {
        Dimension min;
        Dimension pref;
        Dimension max;
        float yalign;
        float xalign;
        
        Invalidator(Component comp) {
            this.setLayout(null);
            this.add(comp);
            this.cacheChildSizes();
        }
        
        @Override
        public void invalidate() {
            super.invalidate();
            if (this.getParent() != null) {
                ComponentView.this.preferenceChanged(null, true, true);
            }
        }
        
        @Override
        public void doLayout() {
            this.cacheChildSizes();
        }
        
        @Override
        public void setBounds(int x, int y, int n, int n2) {
            super.setBounds(x, y, n, n2);
            if (this.getComponentCount() > 0) {
                this.getComponent(0).setSize(n, n2);
            }
            this.cacheChildSizes();
        }
        
        public void validateIfNecessary() {
            if (!this.isValid()) {
                this.validate();
            }
        }
        
        private void cacheChildSizes() {
            if (this.getComponentCount() > 0) {
                Component component = this.getComponent(0);
                this.min = component.getMinimumSize();
                this.pref = component.getPreferredSize();
                this.max = component.getMaximumSize();
                this.yalign = component.getAlignmentY();
                this.xalign = component.getAlignmentX();
            }
            else {
                Dimension min = new Dimension(0, 0);
                this.max = min;
                this.pref = min;
                this.min = min;
            }
        }
        
        @Override
        public void setVisible(boolean b) {
            super.setVisible(b);
            if (this.getComponentCount() > 0) {
                this.getComponent(0).setVisible(b);
            }
        }
        
        @Override
        public boolean isShowing() {
            return true;
        }
        
        @Override
        public Dimension getMinimumSize() {
            this.validateIfNecessary();
            return this.min;
        }
        
        @Override
        public Dimension getPreferredSize() {
            this.validateIfNecessary();
            return this.pref;
        }
        
        @Override
        public Dimension getMaximumSize() {
            this.validateIfNecessary();
            return this.max;
        }
        
        @Override
        public float getAlignmentX() {
            this.validateIfNecessary();
            return this.xalign;
        }
        
        @Override
        public float getAlignmentY() {
            this.validateIfNecessary();
            return this.yalign;
        }
        
        @Override
        public Set<AWTKeyStroke> getFocusTraversalKeys(int id) {
            return KeyboardFocusManager.getCurrentKeyboardFocusManager().getDefaultFocusTraversalKeys(id);
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            Boolean b = (Boolean)propertyChangeEvent.getNewValue();
            if (this.getComponentCount() > 0) {
                this.getComponent(0).setEnabled((boolean)b);
            }
        }
    }
}
